
#include "palrt.h"
#include "pal.h"
#include "stdlib.h"
#include "pal_mstypes.h"
#include "pal_error.h"
#include <new>
#include <memory.h>

bool ResizeBuffer(char *&buffer, int& size, int currLen, int newSize, bool &fixedBuffer)
{
	newSize *= 1.5;
	_ASSERTE(newSize > size); // check for overflow

    if (newSize < 32)
        newSize = 32;

	char *newBuffer = new char[newSize];

	memcpy(newBuffer, buffer, currLen);

	if (!fixedBuffer)
		delete[] buffer;

	buffer = newBuffer;
	size = newSize;
	fixedBuffer = false;

	return true;
}

bool WriteToBuffer(const BYTE *src, int len, char *&buffer, int& offset, int& size, bool &fixedBuffer)
{
    if (!src) return true;
	if (offset + len > size)
	{
		if (!ResizeBuffer(buffer, size, offset, size + len, fixedBuffer))
			return false;
	}

	memcpy(buffer + offset, src, len);
	offset += len;
	return true;
}

bool WriteToBuffer(PCWSTR str, char *&buffer, int& offset, int& size, bool &fixedBuffer)
{
    if (!str) return true;
	int byteCount = (PAL_wcslen(str) + 1) * sizeof(*str);

	if (offset + byteCount > size)
	{
		if (!ResizeBuffer(buffer, size, offset, size + byteCount, fixedBuffer))
			return false;
	}

	memcpy(buffer + offset, str, byteCount);
	offset += byteCount;
	return true;
}

bool WriteToBuffer(const char *str, char *&buffer, int& offset, int& size, bool &fixedBuffer)
{
    if (!str) return true;
	int len = strlen(str) + 1;
	if (offset + len > size)
	{
		if (!ResizeBuffer(buffer, size, offset, size + len, fixedBuffer))
			return false;
	}

	memcpy(buffer + offset, str, len);
	offset += len;
	return true;
}